-- Title: Skill Helper
-- Author: JerichoHM
-- Version: 2.2.221
-- Desc: A simple addon for tracking and using skills

-- GLOBALS
local SKILLCAP = 600
local y = -5
local imgFolder = "Interface\\AddOns\\SkillHelper\\IMAGES\\"
local shFrameBG = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background.blp", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border.blp", tile = true, tileSize = 32, edgeSize = 16, insets = {left = 3, right = 3, top = 3, bottom = 3}}
local shBarBG = { bgFile = "Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar", edgeFile = nil, tile = false, tileSize = 32, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0}}
SLASH_SKILLHELPER1 = '/SHelper'
local icon

--Version Information
local Title = "Skill Helper"
local Core = "2"
local Revision = "2"
local Build = "220"

-- Frame Definitions
local shFrame

-- Skills Table
local skills = {}

-- Frames Table
local bars = {}

function shMain()
	shMainFrame()
end

function shMainFrame()
	-- Create Core UI Frame
	shFrame = CreateFrame("Frame", "shFrame", UIParent)
	shFrame:SetFrameStrata("LOW")
	shFrame:SetPoint("CENTER", UIParent)
	shFrame:SetWidth(170)
	shFrame:SetHeight(28)
	shFrame:SetBackdrop(shFrameBG)

	--Mouse Enabling, Movement constraints, and Movment functions
	shFrame:SetClampedToScreen(true)
	shFrame:EnableMouse(true)
	shFrame:SetMovable(true)
	
	shFrame:RegisterForDrag("LeftButton")
	shFrame:SetScript("OnDragStart", shFrame.StartMoving)
	shFrame:SetScript("OnDragStop", shFrame.StopMovingOrSizing)

	-- Event Handlers
	shFrame:RegisterEvent("SKILL_LINES_CHANGED")
	shFrame:RegisterEvent("ADDON_LOADED")
	shFrame:SetScript("OnEvent", shOnEvent)

	shFrame:Show()
end

function shDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
	local bar = bars[name]
	if not bar then --if bar doesn't exist
		-- The Skill bar
		bar = CreateFrame("StatusBar", nil, shFrame, nil)
		bar:SetFrameStrata("LOW")
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetSize(160,18)
		bar:SetBackdrop(shBarBG)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetStatusBarTexture("Interface\\PaperDollInfoFrame\\UI-Character-Skills-Bar")
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetMinMaxValues(0,1)
		bar:SetValue((rank + rankModifier)/maxRank)

		-- The text on the Skill Bar
		local barTxt = bar:CreateFontString(nil, "ARTWORK")
		isValid = barTxt:SetFont("Fonts\\FRIZQT__.TTF", 10)
		barTxt:SetPoint("CENTER", bar, "CENTER", 0, 1)
		bar.text = barTxt

		-- The first button on the skills action bar
		local barBtn1 = CreateFrame("Button", nil, bar, "SecureActionButtonTemplate")
		barBtn1:SetFrameStrata("LOW")
		barBtn1:SetPoint("TOP", bar, "TOPRIGHT", 13, 0)
		barBtn1:SetSize(18,18)
		barBtn1:EnableMouse(true)
		barBtn1:SetAttribute("type", "spell");
		barBtn1:SetAttribute("spell", shGetSpell(name, 1));
		barBtn1BG = { bgFile = imgFolder .. shGetButton(name, 1) .. ".BLP", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
		barBtn1:SetBackdrop(barBtn1BG)
		bar.btn1 = barBtn1

		-- The second button on the skills action bar if applicable
		if numSpells == 2 then
			if name ~= "Skinning" and name ~= "Tailoring" and name ~= "Alchemy" and name ~= "Engineering" and name ~= "Blacksmithing" and name ~= "Leatherworking" and name ~= "Mining" then
				local barBtn2 = CreateFrame("Button", nil, barBtn1, "SecureActionButtonTemplate")
				barBtn2:SetFrameStrata("LOW")
				barBtn2:SetPoint("TOP", barBtn1, "TOPRIGHT", 10, 0)
				barBtn2:SetSize(18,18)
				barBtn2:EnableMouse(true)
				barBtn2:SetAttribute("type", "spell");
				barBtn2:SetAttribute("spell", shGetSpell(name, 2));
				barBtn2BG = { bgFile = imgFolder .. shGetButton(name, 2) .. ".BLP", edgeFile = nil, tile = false, tileSize = 18, edgeSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0}}
				barBtn2:SetBackdrop(barBtn2BG)
				bar.btn2 = barBtn2
			end
		end

		-- Text formatting for the text on the Skill Bar
		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)

		-- Add to the array
		bars[name] = bar
	else --if bar exists
		bar:ClearAllPoints()
	end

	-- if the rank is in trainable range and not skill cap then turn bar red
	if	rank >= maxRank - 25 and maxRank < SKILLCAP then
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetStatusBarColor(0.7, 0.0, 0.0, 1.0)
		bar:SetBackdropColor(0.7, 0.0, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFF0000%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	else -- otherwise keep bar green
		bar:SetPoint("TOP", shFrame, "TOP", 0, y)
		bar:SetStatusBarColor(0.0, 0.7, 0.0, 1.0)
		bar:SetBackdropColor(0.0, 0.7, 0.0, 0.5)
		bar:SetValue((rank + rankModifier)/maxRank)

		bar.text:SetFormattedText("|cFFFFFF00%s: |cFFFFFFFF%s+%s/%s", name, rank, rankModifier, maxRank)
	end

	shFrame:SetHeight((y - 23)* -1)
end

function shUpdateData()
	-- Capture the SkillIndes for the players Professions
	local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()

	-- Build an array of the Professions
	local profs = {prof1, prof2, archaeology, fishing, cooking, firstAid}

	for k, v in pairs(profs) do
		-- Fetch the details for each Profession
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine, rankModifier = GetProfessionInfo(v)
		shDrawBar(name, rank, rankModifier, maxRank, numSpells, y)
		y = y - 18

		table.insert( skills, name)
	end

	shCleanUp()

	y = -5
end

function shCleanUp()
	local match = false
	local clnBar

	for k, d in pairs(bars) do -- cycle bar names
		for i, v in ipairs(skills) do -- cycle skill nams
			if k == v then -- if the names match set true
				match = true
				break
			else -- otherwise false
				match = false
			end
		end

		-- store bar to local variable
		clnBar = bars[k]

		if match == false then -- If bar and skill don't match hide bar and set to false
			clnBar:Hide()
			match = false
		elseif match == true then -- If bar and skill match show
			clnBar:Show()
		end
	end

	while ((# skills) > 0) do -- cycle through number of skills
		table.remove( skills, 1 ) -- remove skills from the table
	end
end

function shOnEvent(self, event, arg1)
	if event == "SKILL_LINES_CHANGED" then
		if not InCombatLockdown() then
			shUpdateData()
		end
	elseif event == "ADDON_LOADED" and arg1 == "SkillHelper" then
		if shAutohide == nil then
			shAutohide = false
		end
		if shLock == nil then
			shLock = true
			shFrame:EnableMouse(true)
		elseif shLock == true then
			shFrame:EnableMouse(true)
		elseif shLock == false then
			shFrame:EnableMouse(false)
		end
		if shHidden == nil then
			shHidden = false
			shFrame:Show()
		elseif shHidden == false then
			shFrame:Show()
		elseif shHidden == true then
			shFrame:Hide()
		end
		
		if shDB == nil then
			shDB = { hide = false }
		end
		
		shMiniMap();
		
		if shDB.hide then
			icon:Hide("SkillHelper")
			shDB.hide = true
		else
			icon:Show("SkillHelper")
			shDB.hide = false
		end
	end
end

function shGetButton(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Campfire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "FA"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospect"
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return "Smelt"
		--elseif spellNum == 2 then
			--return "Smelt"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

function shGetSpell(name, spellNum)
	if name == "Archaeology" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Survey"
		end
	elseif name == "Cooking" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Cooking Fire"
		end
	elseif name == "Fishing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "First Aid" then
		if spellNum == 1 then
			return "First Aid"
		end
	elseif name == "Herbalism" then
		if spellNum == 1 then
			return "Herb Gathering"
		elseif spellNum == 2 then
			return "Lifeblood"
		end
	elseif name == "Inscription" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Milling"
		end
	elseif name == "Jewelcrafting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Prospecting"
		end
	elseif name == "Mining" then
		if spellNum == 1 then
			return "Smelting"
		--elseif spellNum == 2 then
			--return "Smelting"
		end
	elseif name == "Skinning" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "MoA"
		end
	elseif name == "Alchemy" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Blacksmithing" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Enchanting" then
		if spellNum == 1 then
			return name
		elseif spellNum == 2 then
			return "Disenchant"
		end
	elseif name == "Engineering" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Leatherworking" then
		if spellNum == 1 then
			return name
		end
	elseif name == "Tailoring" then
		if spellNum == 1 then
			return name
		end
	end
end

function SlashCmdList.SKILLHELPER(msg)
	if msg == "toggle" then
		shToggle()
	elseif msg == "lock" then
		shLocker()
	elseif msg == "reset" then
		shReset()
	elseif msg == "info" then
		shInfo()
	elseif msg == "mmtoggle" then
		shMMToggle()
	else
		print("Skill Helper Slash Commands")
		print("type /SHelper toggle to toggle the addon hidden state")
		print("type /SHelper lock to toggle locking")
		print("type /SHelper reset to reset to default position")
		print("type /SHelper mmtoggle to toggle the minimap button on/off")
		print("tpye /SHelper info to view current build information")
	end
end

function shToggle()
	-- true for it is hidden and false for it isn't hidden
	if shHidden == false then
		shFrame:Hide()
		print("Skill Helper hidden")
		shHidden = true
	elseif shHidden == true then
		shFrame:Show()
		print("Skill Helper visible")
		shHidden = false
	end
end

function shLocker()
	-- Remember shLock is backwards. true for unlocked and false for locked
	if shLock == true then
		shLock = false
		shFrame:EnableMouse(shLock)
		print("Skill Helper locked")
	elseif shLock == false then
		shLock = true
		shFrame:EnableMouse(shLock)
		print("Skill Helper unlocked")
	end
end

function shReset()
	shFrame:SetPoint("CENTER", UIParent, "CENTER")
	shLock = true
	shFrame:EnableMouse(shLock)
	shFrame:Show()
	shHidden = false
	print("Skill Helper Reset")
end

function shInfo()
	print(Title .. " " .. Core .. "." .. Revision .. "." .. Build)
end

function shMMToggle()
	if shDB.hide then
		icon:Show("SkillHelper")
		shDB.hide = false
		print("Skill Helper Minimap Icon visible")
	else
		icon:Hide("SkillHelper")
		shDB.hide = true
		print("Skill Helper Minimap Icon hidden")
	end
end

function shMiniMap()
	local ldb = LibStub("LibDataBroker-1.1", true)
	if not ldb then return end
	local shLDB = ldb:NewDataObject("SkillHelper", {
		type = "launcher",
		icon = "Interface\\AddOns\\SkillHelper\\IMAGES\\icon",
		OnClick = function(clickedframe, button)
			if button == "RightButton" then
				if IsShiftKeyDown() then
					shMMToggle()
				else
					shLocker()
				end
			elseif button == "MiddleButton" then
				shReset()
			else
				shToggle()
			end
		end,
		OnTooltipShow = function(tt)
			tt:AddLine(Title .. " " .. Core .. "." .. Revision .. "." .. Build)
			tt:AddLine("|r  Left-Click to toggle window")
			tt:AddLine("|r  Right-Click to toggle lock")
			tt:AddLine("|r  Shift+Right-Click to toggle the minimap icon")
			tt:AddLine("|r  Middle-Click to reset")
		end,
	})
	if shLDB then
		icon = LibStub("LibDBIcon-1.0", true)
		if not icon then return end
		icon:Register("SkillHelper", shLDB, shDB)
	end
end

shMain()
